/// <reference path="../../typings/tsd.d.ts"/>
import * as React from "react";
import * as RN from "react-native";
import { View } from "./View";
import { PixelLine } from "./PixelLine";
import { ScrollView } from "./ScrollView";
import { Font } from "./Font";
import { TextAlignment } from "./Enum";
import { PresetColor } from "./Types";
import { TapGestureRecognizer } from "./GestureRecognizer";
/**
 * Label
 */
export class NavigationBar extends React.Component {
    render() {
        let itemsView = this.props.navigationItems.map((element, idx) => {
            let leftStyle = {};
            let titleStyle = {};
            let rightStyle = {};
            if (this.props.navigationItems.length > 1) {
                if (idx == this.props.navigationItems.length - 1) {
                    Object.assign(leftStyle, this.props.navigationController.transitionCoordinator.nextBarItemsStyle());
                    Object.assign(titleStyle, this.props.navigationController.transitionCoordinator.nextBarItemsStyle(), this.props.navigationController.transitionCoordinator.nextBarCenterItemStyle());
                    Object.assign(rightStyle, this.props.navigationController.transitionCoordinator.nextBarItemsStyle());
                }
                else if (idx == this.props.navigationItems.length - 2) {
                    Object.assign(leftStyle, this.props.navigationController.transitionCoordinator.previousBarItemsStyle());
                    Object.assign(titleStyle, this.props.navigationController.transitionCoordinator.previousBarItemsStyle(), this.props.navigationController.transitionCoordinator.previousBarCenterItemStyle());
                    Object.assign(rightStyle, this.props.navigationController.transitionCoordinator.previousBarItemsStyle());
                }
                else {
                    leftStyle.opacity = 0.0;
                    titleStyle.opacity = 0.0;
                    rightStyle.opacity = 0.0;
                }
            }
            else {
                leftStyle.opacity = 1.0;
                titleStyle.opacity = 1.0;
                rightStyle.opacity = 1.0;
            }
            return React.createElement(NavigationItemView, {key: "iv_" + idx, userInteractionEnabled: true, frame: { x: 0, y: 0, mx: 0, my: 0 }, navigationItem: element, leftStyle: leftStyle, titleStyle: titleStyle, rightStyle: rightStyle});
        });
        return (React.createElement(RN.View, null, React.createElement(View, {userInteractionEnabled: true, frame: { x: 0, y: 0, mx: 0, height: 20 }, backgroundColor: this.props.barTintColor, gestureRecognizers: [new TapGestureRecognizer((state) => {
                ScrollView.scrollsToTop();
            })]}), React.createElement(View, {userInteractionEnabled: true, animatable: true, frame: { x: 0, y: (this.props.statusBarHidden ? 0 : 20), mx: 0, height: 44 }, backgroundColor: this.props.barTintColor}, itemsView, React.createElement(PixelLine, {color: this.props.separatorColor, bottom: 0}))));
    }
}
NavigationBar.defaultProps = Object.assign({}, View.defaultProps, {
    titleLabelProps: { textColor: "black", font: Font.boldSystemFontOfSize(17.0), textAlignment: TextAlignment.Center, numberOfLines: 1 },
    tintColor: PresetColor.tintColor,
    barTintColor: "#f9f9f9",
    separatorColor: PresetColor.lineColor,
    backIndicatorImage: require('./Assets/ux_back.png'),
    statusBarHidden: false,
    navigationItems: [],
});
class NavigationItemView extends View {
    view() {
        return (React.createElement(RN.View, {style: { flex: 1, flexDirection: "row" }, pointerEvents: "auto"}, React.createElement(RN.Animated.View, {style: [{ position: "absolute", left: 0, top: 0, bottom: 0, right: 0, alignItems: "center" }, this.props.titleStyle]}, this.props.navigationItem.titleView), React.createElement(RN.Animated.View, {style: [{ position: "absolute", left: 0, top: 0, bottom: 0 }, this.props.leftStyle], renderToHardwareTextureAndroid: true}, this.props.navigationItem.leftBarButtonItem || this.props.navigationItem.backBarButtonItem), React.createElement(RN.Animated.View, {style: [{ position: "absolute", right: 0, top: 0, bottom: 0 }, this.props.rightStyle]}, this.props.navigationItem.rightBarButtonItem)));
    }
}
