/// <reference path="../../typings/tsd.d.ts"/>
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import * as React from "react";
import * as RN from "react-native";
import { ViewController } from "./ViewController";
import { NavigationBar } from "./NavigationBar";
import { NavigationItem } from "./NavigationItem";
import { StatusBar } from "./StatusBar";
import { NavigationTransitionCoordinator } from "./NavigationTransitionCoordinator";
/**
 * NavigationController
 */
export class NavigationController extends ViewController {
    constructor(props) {
        super(props);
        this.barProps = {};
        this._rootViewController = undefined;
        this._viewControllers = [];
        this._navigationItems = [];
        this._transitionProgressing = false;
        this._transitionCoordinator = new NavigationTransitionCoordinator();
        // Gestures
        this.panResponder = RN.PanResponder.create({
            onStartShouldSetPanResponder: () => false,
            onStartShouldSetPanResponderCapture: () => false,
            onMoveShouldSetPanResponder: (event, state) => {
                if (this._transitionProgressing) {
                    return false;
                }
                if (this.viewControllers.length <= 1) {
                    return false;
                }
                if (event.nativeEvent.pageX < 44.0 && state.dx > 0) {
                    return true;
                }
                return false;
            },
            onMoveShouldSetPanResponderCapture: (event, state) => {
                if (this._transitionProgressing) {
                    return false;
                }
                if (this.viewControllers.length <= 1) {
                    return false;
                }
                if (event.nativeEvent.pageX < 44.0 && state.dx > 0) {
                    return true;
                }
                return false;
            },
            onPanResponderGrant: () => { },
            onPanResponderMove: (event, state) => {
                this._transitionCoordinator.step.setValue(1.0 - (event.nativeEvent.pageX / this.bounds.width));
            },
            onPanResponderTerminationRequest: () => {
                return false;
            },
            onPanResponderRelease: (event, state) => {
                this._transitionCoordinator.step.stopAnimation((step) => {
                    if (step < 0.5 && state.vx >= 0.0) {
                        this.popViewController(true, Math.abs(state.vx));
                    }
                    else if (state.vx > 0.2) {
                        this.popViewController(true, Math.abs(state.vx));
                    }
                    else {
                        this._transitionCoordinator.forward(() => { });
                    }
                });
            }
        });
        this.rootViewController = props.rootViewController;
    }
    get transitionCoordinator() {
        return this._transitionCoordinator;
    }
    viewDidLoad() {
        super.viewDidLoad();
        RN.BackAndroid.addEventListener('hardwareBackPress', () => {
            if (this.viewControllers.length <= 1) {
                return false;
            }
            this.popViewController();
            return true;
        });
    }
    viewWillLayoutSubviews() {
        super.viewWillLayoutSubviews();
        this._transitionCoordinator.boundsWidth = this.bounds.width;
    }
    topHeight() {
        if (StatusBar.isHidden) {
            return 44.0;
        }
        return 64.0;
    }
    loadView() {
        this.viewControllers.forEach((element, idx) => {
            element.props.navigationIndex = idx;
            element.props.navigationController = this;
        });
        let vcs = this.viewControllers.map((item, idx) => {
            if (this.viewControllers.length > 1 && idx == this.viewControllers.length - 1) {
                return (React.createElement(RN.Animated.View, {key: "navi_vc_" + idx, style: [{ position: "absolute", top: this.topHeight(), left: 0, bottom: 0, right: 0, backgroundColor: "white" }, this._transitionCoordinator.nextViewControllerStyle()]}, item));
            }
            else if (this.viewControllers.length > 1 && idx == this.viewControllers.length - 2) {
                return (React.createElement(RN.Animated.View, {key: "navi_vc_" + idx, style: [{ position: "absolute", top: this.topHeight(), left: 0, bottom: 0, right: 0, backgroundColor: "white" }, this._transitionCoordinator.previousViewControllerStyle()]}, item, React.createElement(RN.Animated.View, {style: [{ position: "absolute", top: 0, left: 0, bottom: 0, right: 0, backgroundColor: "#00000010" }, this._transitionCoordinator.previousViewControllerMaskStyle()]})));
            }
            else {
                return (React.createElement(RN.Animated.View, {key: "navi_vc_" + idx, style: { position: "absolute", top: this.topHeight(), left: 0, bottom: 0, right: 0, marginLeft: 0, backgroundColor: "white" }}, item));
            }
        });
        return (React.createElement(RN.View, __assign({style: { flex: 1, backgroundColor: "white" }}, this.panResponder.panHandlers), vcs, React.createElement(NavigationBar, __assign({ref: "NavigationBar", statusBarHidden: StatusBar.isHidden, navigationItems: this._navigationItems, navigationController: this}, this.barProps))));
    }
    showViewController(item, animated = true) {
        if (this._transitionProgressing) {
            return;
        }
        if (this._navigationItems.length > 0) {
            item.props.backTitle = this._navigationItems[this._navigationItems.length - 1].title;
        }
        this.viewControllers.push(item);
        this._transitionCoordinator.step.setValue(0.0);
        this.forceUpdate(() => {
            if (animated) {
                this._transitionProgressing = true;
                ViewController.secondQueue.push((completionBlock) => {
                    this._transitionCoordinator.forward(() => {
                        this._transitionProgressing = false;
                        completionBlock();
                    });
                });
                setTimeout(() => {
                    ViewController.digestQueue();
                }, 100);
            }
            else {
                this._transitionCoordinator.step.setValue(1.0);
            }
        });
    }
    popViewController(animated = true, velocity = 0.0) {
        if (this._transitionProgressing) {
            return;
        }
        if (this.viewControllers.length <= 1) {
            return;
        }
        if (animated) {
            let preventing = false;
            this._transitionProgressing = true;
            this._transitionCoordinator.backward(() => {
                if (preventing) {
                    return;
                }
                preventing = true;
                this._transitionProgressing = false;
                let viewController = this.viewControllers.pop();
                viewController.props.navigationController = undefined;
                this._navigationItems.pop();
                this.forceUpdate();
            }, velocity);
        }
        else {
            setTimeout(() => {
                this._transitionCoordinator.step.setValue(0.0);
                let viewController = this.viewControllers.pop();
                viewController.props.navigationController = undefined;
                this._navigationItems.pop();
                this.forceUpdate();
                this._transitionCoordinator.step.setValue(1.0);
            });
        }
    }
    popToRootViewController(animated = true) {
        if (this._transitionProgressing) {
            return;
        }
        if (this.viewControllers.length <= 1) {
            return;
        }
        else if (this.viewControllers.length <= 2) {
            this.popViewController(animated);
        }
        else {
            this.viewControllers = this.viewControllers.filter((element, idx) => {
                if (idx == 0 || idx == this.viewControllers.length - 1) {
                    return true;
                }
                else {
                    return false;
                }
            });
            this._navigationItems = this._navigationItems.filter((element, idx) => {
                if (idx == 0 || idx == this._navigationItems.length - 1) {
                    return true;
                }
                else {
                    return false;
                }
            });
            this.forceUpdate();
            this.popViewController(animated);
        }
    }
    resetNavigationItems(navigationIndex, navigationItem) {
        if (typeof navigationIndex === "number" && navigationItem instanceof NavigationItem && navigationIndex < this.viewControllers.length) {
            this._navigationItems[navigationIndex] = navigationItem;
            this.forceUpdate();
        }
    }
    // Setter & Getter
    get rootViewController() {
        return this._rootViewController;
    }
    set rootViewController(item) {
        this._rootViewController = item;
        this.viewControllers = [this._rootViewController];
    }
    get viewControllers() {
        return this._viewControllers;
    }
    set viewControllers(items) {
        this._viewControllers = items;
        this.forceUpdate();
    }
}
