/// <reference path="../../typings/tsd.d.ts"/>
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import * as React from "react";
import { View } from "./View";
import { Image } from "./ImageView";
import { TextButton } from "./Button";
import { Label } from "./Label";
import { Font } from "./Font";
import { ImageRenderingMode, ControlEvents } from "./Enum";
/**
 * BarButtonItem
 */
export class BarButtonItem extends View {
    static backItem(handler, title, indicatorImage, tintColor) {
        let image = indicatorImage ? indicatorImage : Image.resource(require('./Assets/ux_back.png')).imageWithRenderingMode(ImageRenderingMode.Template);
        return React.createElement(BarButtonItem, {image: image, title: title && title.length < 4 ? title : "返回", middleInset: 4.0, tintColor: tintColor, handler: handler});
    }
    render() {
        return (React.createElement(TextButton, {margin: { left: 8, right: 8 }, middleInset: this.props.middleInset, style: { flex: 1, paddingLeft: 0, paddingRight: 0 }, title: this.props.title, image: this.props.image, tintColor: this.props.tintColor, titleLabelProps: { font: Font.fontWithSize(16.0) }, backTouchingArea: this.props.backTouchingArea, handler: this.props.handler}));
    }
}
BarButtonItem.defaultProps = Object.assign({}, View.defaultProps, {
    userInteractionEnabled: true,
    title: null,
    image: null,
    middleInset: 0.0,
    backTouchingArea: 0.0,
    handler: () => { }
});
/**
 * NavigationItem
 */
export class NavigationItem {
    constructor(viewController) {
        this.viewController = viewController;
    }
    get titleView() {
        return this._titleView;
    }
    set titleView(titleView) {
        this._titleView = titleView;
        this.forceUpdateNavigationBar();
    }
    get backBarButtonItem() {
        return this._backBarButtonItem;
    }
    set backBarButtonItem(item) {
        this._backBarButtonItem = item;
        this.forceUpdateNavigationBar();
    }
    get leftBarButtonItem() {
        return this._leftBarButtonItem;
    }
    set leftBarButtonItem(item) {
        this._leftBarButtonItem = item;
        this.forceUpdateNavigationBar();
    }
    get rightBarButtonItem() {
        return this._rightBarButtonItem;
    }
    set rightBarButtonItem(item) {
        this._rightBarButtonItem = item;
        this.forceUpdateNavigationBar();
    }
    forceUpdateNavigationBar() {
        if (this.viewController && this.viewController.navigationController) {
            this.viewController.navigationController.resetNavigationItems(this.viewController.props.navigationIndex, this);
        }
    }
    barProps() {
        if (this.viewController && this.viewController.navigationController) {
            return this.viewController.navigationController.barProps;
        }
        return {};
    }
    resetTitleView(title) {
        this.titleView = (React.createElement(Label, __assign({frame: { cx: 0, cy: 0 }, text: title, font: Font.boldSystemFontOfSize(17.0)}, this.barProps().titleLabelProps)));
    }
    resetBackButtonItem(title, navigationController) {
        this.backBarButtonItem = BarButtonItem.backItem((event) => {
            if (event == ControlEvents.TouchUpInside) {
                navigationController.popViewController();
            }
        }, title, this.barProps().backIndicatorImage, this.barProps().tintColor);
    }
}
