import * as RN from "react-native";
/**
 * NavigationTransitionCoordinator
 */
export class NavigationTransitionCoordinator {
    constructor() {
        this.boundsWidth = 0.0;
        this.step = new RN.Animated.Value(0.0);
        // ViewController Transition
        this.nextViewControllerMarginLeftValue = new RN.Animated.Value(0.0);
        this.nextViewControllerMarginRightValue = new RN.Animated.Value(0.0);
        this.previousViewControllerMarginLeftValue = new RN.Animated.Value(0.0);
        this.previousViewControllerMarginRightValue = new RN.Animated.Value(0.0);
        this.previousViewControllerMaskAlphaValue = new RN.Animated.Value(0.0);
        // NavigationBar Transition
        this.nextBarItemsAlphaValue = new RN.Animated.Value(0.0);
        this.nextBarCenterItemMarginLeftValue = new RN.Animated.Value(0.0);
        this.previousBarItemsAlphaValue = new RN.Animated.Value(0.0);
        this.previousBarCenterItemMarginLeftValue = new RN.Animated.Value(0.0);
        this.step.addListener((state) => {
            this.resetWithStep(state.value);
        });
    }
    forward(completionBlock) {
        RN.Animated.spring(this.step, {
            toValue: 1.0,
            bounciness: 1.0,
            speed: 16.0,
        }).start(completionBlock);
    }
    backward(completionBlock, velocity = 0.0) {
        RN.Animated.spring(this.step, {
            toValue: 0.0,
            bounciness: 1.0,
            speed: 24.0 * (1.0 + velocity),
        }).start(() => {
            completionBlock();
            this.step.setValue(1.0);
        });
    }
    resetWithStep(step) {
        this.nextViewControllerMarginLeftValue.setValue((1 - step) * this.boundsWidth);
        this.nextViewControllerMarginRightValue.setValue(-((1 - step) * this.boundsWidth));
        this.previousViewControllerMarginLeftValue.setValue(-(this.boundsWidth * 0.50) * step);
        this.previousViewControllerMarginRightValue.setValue((this.boundsWidth * 0.50) * step);
        this.previousViewControllerMaskAlphaValue.setValue(step);
        this.nextBarItemsAlphaValue.setValue(step);
        this.nextBarCenterItemMarginLeftValue.setValue(this.boundsWidth / 2.0 * (1.0 - step));
        this.previousBarItemsAlphaValue.setValue(1.0 - step);
        this.previousBarCenterItemMarginLeftValue.setValue(-this.boundsWidth / 2.0 * step);
    }
    reset() {
        this.step.stopAnimation((step) => {
            this.nextViewControllerMarginLeftValue.setValue((1 - step) * this.boundsWidth);
            this.nextViewControllerMarginRightValue.setValue(-((1 - step) * this.boundsWidth));
            this.previousViewControllerMarginLeftValue.setValue(-(this.boundsWidth * 0.50) * step);
            this.previousViewControllerMarginRightValue.setValue((this.boundsWidth * 0.50) * step);
            this.previousViewControllerMaskAlphaValue.setValue(step);
            this.nextBarItemsAlphaValue.setValue(step);
            this.nextBarCenterItemMarginLeftValue.setValue(this.boundsWidth / 2.0 * (1.0 - step));
            this.previousBarItemsAlphaValue.setValue(1.0 - step);
            this.previousBarCenterItemMarginLeftValue.setValue(-this.boundsWidth / 2.0 * step);
        });
    }
    nextViewControllerStyle() {
        return {
            left: this.nextViewControllerMarginLeftValue,
            right: this.nextViewControllerMarginRightValue,
        };
    }
    previousViewControllerStyle() {
        return {
            left: this.previousViewControllerMarginLeftValue,
            right: this.previousViewControllerMarginRightValue,
        };
    }
    previousViewControllerMaskStyle() {
        return {
            opacity: this.previousViewControllerMaskAlphaValue
        };
    }
    nextBarItemsStyle() {
        return {
            opacity: this.nextBarItemsAlphaValue
        };
    }
    nextBarCenterItemStyle() {
        return {
            left: this.nextBarCenterItemMarginLeftValue
        };
    }
    previousBarItemsStyle() {
        return {
            opacity: this.previousBarItemsAlphaValue
        };
    }
    previousBarCenterItemStyle() {
        return {
            left: this.previousBarCenterItemMarginLeftValue
        };
    }
}
