import * as React from "react";
import * as RN from "react-native";
import { Control } from "./Control";
import { ScrollView } from "./ScrollView";
import { Label } from "./Label";
import { ImageView, Image } from "./ImageView";
import { Font } from "./Font";
import { ControlEvents, TextAlignment, ImageRenderingMode } from "./Enum";
const timeFont = Font.fontWithSize(11.0);
timeFont.familyName = "monospace";
timeFont.fontWeight = "500";
const textFont = Font.fontWithSize(11.0);
timeFont.familyName = "monospace";
/**
 * StatusBar
 */
export class StatusBar extends React.Component {
    constructor(props) {
        super(props);
        this.alphaValue = new RN.Animated.Value(1.0);
        this.topValue = new RN.Animated.Value(0.0);
        this.state = StatusBar.sharedInfo || {
            timeString: "",
            batteryPercentage: 1.0,
            ispName: "",
            usingWifi: false,
        };
    }
    setState(state) {
        super.setState(state);
        StatusBar.sharedInfo = this.state;
    }
    componentDidMount() {
        this.subscription = RN.NativeAppEventEmitter.addListener('StatusBarTapped', () => {
            ScrollView.scrollsToTop();
        });
        if (StatusBar.sharedStatusBar === undefined || this.props.shared) {
            StatusBar.sharedStatusBar = this;
            this.androidUpdate();
            this.updateTimer = setInterval(this.androidUpdate.bind(this), 5000);
        }
        this.resetStatusBar(this.props);
    }
    componentWillUnmount() {
        this.subscription.remove();
        if (StatusBar.sharedStatusBar === this) {
            StatusBar.sharedStatusBar = undefined;
        }
        clearInterval(this.updateTimer);
    }
    componentWillReceiveProps(props) {
        this.resetStatusBar(props);
    }
    resetStatusBar(props) {
        if (RN.Platform.OS === "android") {
            if (StatusBar.sharedStatusBar) {
                if (props.animated) {
                    if (props.showHideTransition === "slide") {
                        RN.Animated.timing(StatusBar.sharedStatusBar.topValue, {
                            toValue: props.hidden ? -20.0 : 0.0,
                            duration: 500,
                        }).start();
                    }
                    else {
                        RN.Animated.timing(StatusBar.sharedStatusBar.alphaValue, {
                            toValue: props.hidden ? 0.0 : 1.0,
                            duration: 500,
                        }).start();
                    }
                }
                else {
                    if (props.showHideTransition === "slide") {
                        StatusBar.sharedStatusBar.topValue.setValue(props.hidden ? -20.0 : 0.0);
                    }
                    else {
                        StatusBar.sharedStatusBar.alphaValue.setValue(props.hidden ? 0.0 : 1.0);
                    }
                }
            }
        }
        else {
            this.forceUpdate();
        }
    }
    render() {
        if (StatusBar.sharedStatusBar !== undefined && StatusBar.sharedStatusBar != this) {
            return null;
        }
        if (RN.Platform.OS == "android") {
            return (React.createElement(RN.View, {style: { position: 'absolute', left: 0, right: 0, top: 0, height: 20 }}, React.createElement(RN.StatusBar, {hidden: true}), this.androidStatusBar()));
        }
        else {
            return React.createElement(RN.StatusBar, {barStyle: this.props.barStyle, hidden: this.props.hidden, animated: this.props.animated, showHideTransition: this.props.showHideTransition});
        }
    }
    // android status bar
    androidTime() {
        let date = new Date();
        let hour = date.getHours().toString();
        let minute = date.getMinutes().toString();
        if (hour.length == 1) {
            hour = "0" + hour;
        }
        if (minute.length == 1) {
            minute = "0" + minute;
        }
        return hour + ":" + minute;
    }
    androidBattary(callback) {
        var statusBarInfoManager = RN.NativeModules.StatusBarInfoManager;
        statusBarInfoManager.battery((val) => {
            callback(val);
        });
    }
    androidISPName(callback) {
        var statusBarInfoManager = RN.NativeModules.StatusBarInfoManager;
        statusBarInfoManager.ispName((val) => {
            callback(val);
        });
    }
    androidUsingWIFI(callback) {
        var statusBarInfoManager = RN.NativeModules.StatusBarInfoManager;
        statusBarInfoManager.usingWifi((val) => {
            callback(val);
        });
    }
    androidStatusBar() {
        let tintColor = this.props.barStyle == "default" ? "black" : "white";
        let wifiView = null;
        if (this.state.usingWifi) {
            wifiView = React.createElement(ImageView, {image: Image.resource(require("./Assets/ux_wifi.png")).imageWithRenderingMode(ImageRenderingMode.Template), style: { paddingLeft: 4.0, paddingTop: 0.5 }, tintColor: tintColor});
        }
        return (React.createElement(RN.Animated.View, {style: { flexDirection: "row", height: 20, marginTop: this.topValue, alignItems: "center", opacity: this.alphaValue }}, React.createElement(RN.View, {style: { flex: 1, paddingLeft: 6, flexDirection: "row", alignItems: "center" }}, React.createElement(Label, {text: this.state.ispName, textAlignment: TextAlignment.Left, font: textFont}), wifiView), React.createElement(Label, {text: this.state.timeString, textColor: tintColor, textAlignment: TextAlignment.Center, font: timeFont, style: { flex: 1 }}), React.createElement(RN.View, {style: { flex: 1, paddingRight: 4 }}, React.createElement(RN.View, {style: { alignSelf: "flex-end" }}, React.createElement(ImageView, {image: Image.resource(require("./Assets/ux_battery.png")).imageWithRenderingMode(ImageRenderingMode.Template), tintColor: tintColor}), React.createElement(RN.View, {style: { position: "absolute", left: 1, top: 1, bottom: 1, width: 20.5 * this.state.batteryPercentage, borderRadius: 1.0, backgroundColor: tintColor }}))), React.createElement(Control, {frame: { x: 0, y: 0, mx: 0, my: 0 }, handler: (event) => {
            if (event == ControlEvents.TouchUpInside) {
                ScrollView.scrollsToTop();
            }
        }})));
    }
    androidUpdate() {
        if (RN.Platform.OS !== "android") {
            return;
        }
        this.setState({
            timeString: this.androidTime()
        });
        this.androidISPName((ispName) => {
            this.setState({
                ispName: ispName
            });
        });
        this.androidBattary((value) => {
            this.setState({
                batteryPercentage: value
            });
        });
        this.androidUsingWIFI((value) => {
            this.setState({
                usingWifi: value
            });
        });
    }
}
StatusBar.isHidden = false;
StatusBar.defaultProps = {
    shared: false,
    barStyle: "default",
    hidden: false,
    animated: true,
    showHideTransition: "slide",
};
StatusBar.sharedStatusBar = undefined;
StatusBar.sharedInfo = undefined;
