/// <reference path="../../typings/tsd.d.ts"/>
import * as React from "react";
import * as RN from "react-native";
import { View } from "./View";
import { InterfaceOrientation } from "./Enum";
import { StatusBar } from "./StatusBar";
import { NavigationItem } from "./NavigationItem";
import { ActivityIndicatorView } from "./ActivityIndicatorView";
/**
 * ViewController
 */
export class ViewController extends React.Component {
    constructor(...args) {
        super(...args);
        this.navigationItem = new NavigationItem(this);
        this.lazyLoad = false;
        this.bounds = { width: 0, height: 0 };
        this.loaded = false;
        // Status Bar
        this._statusBar = undefined;
        // auto release
        this.autoReleaseInstance = [];
        // title
        this._title = "";
        this.presented = false;
    }
    static digestQueue() {
        if (this.firstQueue.length > 0) {
            this.firstQueue.shift()(() => {
                this.digestQueue();
            });
        }
        else if (this.secondQueue.length > 0) {
            this.secondQueue.shift()(() => {
                this.digestQueue();
            });
        }
    }
    get navigationController() {
        return this.props.navigationController;
    }
    loadView() {
        return React.createElement(RN.View, null);
    }
    loadingView() {
        return React.createElement(ActivityIndicatorView, {frame: { cx: 0, cy: 0 }, animating: true, tintColor: "#AAAAAA"});
    }
    viewDidLoad() {
    }
    viewWillUnload() {
    }
    viewWillLayoutSubviews() {
        if (RN.Platform.OS === "android") {
            if (this.bounds.width == RN.Dimensions.get("window").width && this.bounds.height == RN.Dimensions.get("window").height) {
                if (RN.Dimensions.get("window").width < RN.Dimensions.get("window").height) {
                    this.willRotateToInterfaceOrientation(InterfaceOrientation.Portrait);
                }
                else {
                    this.willRotateToInterfaceOrientation(InterfaceOrientation.Landscape);
                }
            }
            else if (this.bounds.width == RN.Dimensions.get("window").height && this.bounds.height == RN.Dimensions.get("window").width) {
                if (RN.Dimensions.get("window").width < RN.Dimensions.get("window").height) {
                    this.willRotateToInterfaceOrientation(InterfaceOrientation.Landscape);
                }
                else {
                    this.willRotateToInterfaceOrientation(InterfaceOrientation.Portrait);
                }
            }
        }
        else {
            if (RN.Dimensions.get("window").width < RN.Dimensions.get("window").height) {
                this.willRotateToInterfaceOrientation(InterfaceOrientation.Portrait);
            }
            else {
                this.willRotateToInterfaceOrientation(InterfaceOrientation.Landscape);
            }
        }
    }
    willRotateToInterfaceOrientation(interfaceOrientation) {
        this.setStatusBarHidden(interfaceOrientation == InterfaceOrientation.Landscape, false);
    }
    componentDidMount() {
        this.viewDidLoad();
        RN.BackAndroid.addEventListener('hardwareBackPress', () => {
            if (!!this.props.presentingViewController) {
                this.dismissViewController();
                return true;
            }
            return false;
        });
        if (this.navigationController && this.props.navigationIndex > 0) {
            this.navigationItem.resetBackButtonItem(this.props.backTitle || "返回", this.navigationController);
        }
        if (this.lazyLoad) {
            ViewController.secondQueue.push((completionBlock) => {
                setTimeout(() => {
                    RN.InteractionManager.runAfterInteractions(() => {
                        this.loaded = true;
                        this.forceUpdate(completionBlock);
                    });
                }, 300);
            });
            RN.InteractionManager.runAfterInteractions(() => {
                this.loaded = false;
                this.forceUpdate();
                ViewController.digestQueue();
            });
        }
    }
    render() {
        return (React.createElement(RN.View, {style: [{ marginTop: this.props.marginTop, marginBottom: this.props.marginBottom, flex: 1 }, this.props.style], onLayout: (event) => {
            if (this.bounds.width == event.nativeEvent.layout.width && this.bounds.height == event.nativeEvent.layout.height) {
                return;
            }
            this.bounds = { width: event.nativeEvent.layout.width, height: event.nativeEvent.layout.height };
            this.viewWillLayoutSubviews();
        }}, !this.lazyLoad || this.loaded ? this.loadView() : this.loadingView(), this.modal(), this.statusBar));
    }
    get statusBar() {
        return this._statusBar;
    }
    set statusBar(element) {
        this._statusBar = element;
        StatusBar.isHidden = this._statusBar.props.hidden;
        if (this._statusBar.props.animated) {
            View.setAnimate(() => {
                this.setState({});
            }, 500, 'timing');
        }
        else {
            this.setState({});
        }
    }
    setStatusBarHidden(hidden, animated) {
        if (this.statusBar === undefined) {
            this.statusBar = (React.createElement(StatusBar, {hidden: hidden, animated: animated}));
        }
        else {
            this.statusBar = (React.createElement(StatusBar, {hidden: hidden, animated: animated, showHideTransition: this.statusBar.props.showHideTransition, barStyle: this.statusBar.props.barStyle}));
        }
    }
    autoRelease(obj) {
        this.autoReleaseInstance.push(obj);
    }
    componentWillUnmount() {
        this.viewWillUnload();
        this.autoReleaseInstance.forEach(obj => {
            if (typeof obj === "number") {
                clearTimeout(obj);
                clearInterval(obj);
            }
        });
    }
    get title() {
        return this._title;
    }
    set title(title) {
        this._title = title;
        this.navigationItem.title = this.title;
        this.navigationItem.resetTitleView(this.title);
    }
    presentViewController(viewController, animated = true) {
        viewController.props.presentingViewController = this;
        this.presentedViewController = viewController;
        this.presented = false;
        if (animated) {
            this.setState({}, () => {
                setTimeout(() => {
                    View.setAnimate(() => {
                        this.presented = true;
                        this.setState({});
                    }, 0, 'spring', { friction: 24, tension: 360, toValue: 0.0 }, () => { });
                });
            });
        }
        else {
            this.presented = true;
            this.setState({});
        }
    }
    dismissViewController(animated = true) {
        if (this.presentedViewController !== null && this.presentedViewController !== undefined) {
            this._dismissViewController(animated);
        }
        else if (this.props.presentingViewController !== null && this.props.presentingViewController !== undefined) {
            this.props.presentingViewController._dismissViewController(animated);
        }
    }
    _dismissViewController(animated = true) {
        if (animated) {
            this.presented = false;
            setTimeout(() => {
                View.setLayout(() => {
                    this.setState({});
                }, 500, 'spring', null, () => {
                });
            });
            setTimeout(() => {
                this.presentedViewController = null;
                this.setState({});
            }, 500);
        }
        else {
            this.presented = false;
            this.presentedViewController = null;
            this.setState({});
        }
    }
    modal() {
        if (this.presentedViewController === null || this.presentedViewController === undefined) {
            return null;
        }
        return (React.createElement(View, {frame: { x: 0, y: 0, mx: 0, my: 0 }, margin: { top: !this.presented ? RN.Dimensions.get('window').height : 0.0 }, animatable: true}, this.presentedViewController));
    }
}
ViewController.defaultProps = {
    style: {
        backgroundColor: "white",
        flex: 1,
    }
};
ViewController.firstQueue = [];
ViewController.secondQueue = [];
export class ViewControllerRotationLock {
    static manager() {
        const manager = require('react-native').NativeModules.RotationLockManager;
        if (manager != undefined) {
            return manager;
        }
        else {
            return undefined;
        }
    }
    static lock() {
        if (this.manager()) {
            this.manager().lock();
        }
    }
    static lockOrientation() {
        if (this.manager()) {
            this.manager().lockOrientation();
        }
    }
    static unlock() {
        if (this.manager()) {
            this.manager().unlock();
        }
    }
    static setInterfaceOrientation(interfaceOrientation) {
        if (this.manager()) {
            this.manager().setInterfaceOrientation(interfaceOrientation);
        }
    }
}
